@@ -1,5 +1,5 @@
 from pyg_base import as_list, is_primitive, get_cache, dictattr, Dict, tree_getitem, getargspec, getargs, getcallargs, \
-                    eq, loop, ulist, tree_repr, wrapper, logger, is_strs, is_date, is_num, list_instances, is_ts
+                    eq, loop, ulist, tree_repr, wrapper, logger, is_strs, is_date, is_num, list_instances, is_ts, dictable
                     
 from pyg_cell._dag import get_DAG, get_GAD, add_edge, del_edge, topological_sort 
 
@@ -34,13 +34,21 @@ def cell_output(c):
     if res is None:
         res = _data
     return as_list(res)
-    
+
+
 
 @loop(list, tuple)
 def _cell_item(value, key = None, key_before_data = False):
     if not isinstance(value, cell):
         if isinstance(value, dict):
-            return type(value)(**{k: cell_item(v, key) for k, v in value.items()})
+            try:
+                return type(value)(**{k: cell_item(v, key) for k, v in value.items()})
+            except TypeError:
+                if isinstance(value, dictable):
+                    return type(value)(**{str(k): cell_item(v, key) for k, v in value.items()})
+                else:
+                    return type(value)({k: cell_item(v, key) for k, v in value.items()})
+                
         else:
             return value
     output = cell_output(value)
@@ -70,6 +78,27 @@ def _cell_item(value, key = None, key_before_data = False):
             return Dict(value)[output]
         
 
+@loop(list, tuple)
+def _cell_outputs(c):
+    if isinstance(c, cell):
+        output = cell_output(c)
+        return dictattr({key: c.get(key) for key in output})
+    elif isinstance(c, dict):
+        try:
+            return type(c)(**{k: _cell_outputs(v) for k, v in c.items()})
+        except TypeError:
+            if isinstance(c, dictable):
+                return type(c)(**{str(k): _cell_outputs(v) for k, v in c.items()})
+            else:
+                return type(c)({k: _cell_outputs(v) for k, v in c.items()})
+
+    else:
+        return c
+
+def cell_outputs(value):
+    return _cell_outputs(value)
+    
+
 def cell_clear(value):
     """
     cell_clear clears a cell of its output so that it contains only the essentil stuff to do its calculations.
@@ -100,7 +129,13 @@ def cell_clear(value):
     elif isinstance(value, (tuple, list)):
         return type(value)([cell_clear(v) for v in value])
     elif isinstance(value, dict):
-        return type(value)(**{k : cell_clear(v) for k, v in value.items()})
+        try:
+            return type(value)(**{k : cell_clear(v) for k, v in value.items()})
+        except TypeError:
+            if isinstance(value, dictable):
+                return type(value)(**{str(k): cell_clear(v) for k, v in value.items()})
+            else:
+                return type(value)({k: cell_clear(v) for k, v in value.items()})
     else:
         return value
 
@@ -146,7 +181,13 @@ def _cell_go(value, go, mode = 0):
             return value.go(go = go, mode = mode)
     else:
         if isinstance(value, dict):
-            return type(value)(**{k: _cell_go(v, go = go, mode = mode) for k, v in value.items()})
+            try:
+                return type(value)(**{k: _cell_go(v, go = go, mode = mode) for k, v in value.items()})
+            except TypeError:
+                if isinstance(value, dictable):
+                    return type(value)(**{str(k): _cell_go(v, go = go, mode = mode) for k, v in value.items()})
+                else:
+                    return type(value)({k: _cell_go(v, go = go, mode = mode) for k, v in value.items()})
         else:
             return value
 
@@ -192,7 +233,13 @@ def _cell_load(value, mode):
             return value.load(mode)
     else:
         if isinstance(value, dict):
-            return type(value)(**{k: _cell_load(v, mode) for k, v in value.items()})
+            try:
+                return type(value)(**{k: _cell_load(v, mode) for k, v in value.items()})
+            except TypeError:
+                if isinstance(value, dictable):
+                    return type(value)(**{str(k): _cell_load(v, mode) for k, v in value.items()})
+                else:
+                    return type(value)({k: _cell_load(v, mode) for k, v in value.items()})
         else:
             return value
 
@@ -335,7 +382,42 @@ class cell_func(wrapper):
         called_params.update(called_varkw)
         return res, itemized_varargs, called_params
         
-        
+
+def _is_different_except_cells(v1, v2):
+    """
+    We compare v1 to v2 and check if it is updated..
+    cells are compared at run() check rather than at load
+    """
+    if type(v1)!=type(v2):
+        return True
+    if isinstance(v1, cell):
+        return False
+    if isinstance(v1, (list, tuple)):
+        if len(v1)!=len(v2):
+            return True
+        for a1, a2 in zip(v1, v2):
+            if _is_different_except_cells(a1, a2):
+                return True
+        return False
+    if isinstance(v1, dict):
+        if sorted(v1.keys()) != sorted(v2.keys()):
+            return True
+        for k in v1:
+            if _is_different_except_cells(v1[k], v2[k]):
+                return True
+        return False
+    return not eq(v1, v2)
+
+def _are_any_inputs_updated(new, old):
+    for k, v1 in new._inputs.items():
+        if k not in old: ## k is a new key
+            return True
+        v2 = old[k]
+        if _is_different_except_cells(v1, v2):
+            return True
+    v1, v2 = new.function, old.function
+    return _is_different_except_cells(v1, v2)
+
 
 def is_pairs(pairs):
     """
@@ -487,10 +569,9 @@ class cell(dictattr):
             self_updated = self.get(_updated)
             if is_date(saved_updated) and (self_updated is None or self_updated < saved_updated):
                 output = {key: value for key, value in saved.items() if (key in [_updated] + self._output and value is not None) or key not in self}
-                updated_inputs = [k for k, v in self._inputs.items() if k in saved and v is not None and (type(saved[k])!=type(v) or (is_primitive(v) and saved[k]!=v))]
                 self.update(output)
-                if len(updated_inputs):
-                    self[_updated] = None
+                if _are_any_inputs_updated(self, saved):
+                    self[_updated] = self_updated
         return self
             
     def __call__(self, go = 0, mode = 0, **kwargs):
@@ -956,4 +1037,3 @@ def cell_inputs(value, types = cell):
 
 
 
-
